@@ -7,6 +7,7 @@
  * Authors: Johann Hanne
  *          heavily based on pmt_private.h which was written by
  *          Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
+ *          Jean-Paul Saman <jpsaman@videolan.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -36,40 +37,40 @@
  *****************************************************************************/
 typedef struct dvbpsi_cat_decoder_s
 {
-  dvbpsi_cat_callback           pf_callback;
-  void *                        p_cb_data;
+    DVBPSI_DECODER_COMMON
 
-  dvbpsi_cat_t                  current_cat;
-  dvbpsi_cat_t *                p_building_cat;
+    dvbpsi_cat_callback           pf_cat_callback;
+    void *                        p_cb_data;
 
-  int                           b_current_valid;
-
-  uint8_t                       i_last_section_number;
-  dvbpsi_psi_section_t *        ap_sections [256];
+    dvbpsi_cat_t                  current_cat;
+    dvbpsi_cat_t *                p_building_cat;
 
 } dvbpsi_cat_decoder_t;
 
-
 /*****************************************************************************
- * dvbpsi_GatherCATSections
+ * dvbpsi_cat_sections_gather
  *****************************************************************************
  * Callback for the PSI decoder.
  *****************************************************************************/
-__attribute__((deprecated))
-void dvbpsi_GatherCATSections(dvbpsi_decoder_t* p_decoder,
-                              dvbpsi_psi_section_t* p_section);
-
+void dvbpsi_cat_sections_gather(dvbpsi_t* p_dvbpsi, dvbpsi_psi_section_t* p_section);
 
 /*****************************************************************************
- * dvbpsi_DecodeCATSections
+ * dvbpsi_cat_sections_decode
  *****************************************************************************
  * CAT decoder.
  *****************************************************************************/
+void dvbpsi_cat_sections_decode(dvbpsi_cat_t* p_cat,
+                                dvbpsi_psi_section_t* p_section);
+
+/*****************************************************************************
+ * Deprecated API's
+ *****************************************************************************/
+__attribute__((deprecated))
+void dvbpsi_GatherCATSections(dvbpsi_handle* p_dvbpsi, dvbpsi_psi_section_t* p_section);
 __attribute__((deprecated))
 void dvbpsi_DecodeCATSections(dvbpsi_cat_t* p_cat,
                               dvbpsi_psi_section_t* p_section);
 
-
 #else
 #error "Multiple inclusions of cat_private.h"
 #endif
